'use strict';
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input, } from '@angular/core';
import 'rxjs/add/observable/if';
import 'rxjs/add/operator/take';
import { DFCommonService } from '../services/df.common.api';
import { DFFilterService } from '../services/store/df.filter.service';
import { DFQuestionControlService } from '../services/df.question.control.service';
import { DFStoreFunctions } from '../services/store/df.store.functions';
import { Env } from 'io-ng-util/lib/common/env';
import { Store } from '@ngrx/store';
import { default as cloneDeep } from 'lodash-es/cloneDeep';
import { default as each } from 'lodash-es/each';
import { default as findIndex } from 'lodash-es/findIndex';
import { default as isBoolean } from 'lodash-es/isBoolean';
import { default as isEmpty } from 'lodash-es/isEmpty';
import { default as isEqual } from 'lodash-es/isEqual';
import { default as reduce } from 'lodash-es/reduce';
import { tmpl } from './templates/df.form.pug';
var DFComponent = /** @class */ (function () {
    function DFComponent(env, cd, fs, qcs, store, DSF, DFCS) {
        var _this = this;
        this.questionGroups = [];
        this.formErrors = [];
        this.validations = [];
        this.questions = [];
        this.keyNames = [];
        this.formSubscribe = function () {
            _this.formSubscription = _this.DFCS.form$
                .subscribe(function (options) {
                if (options.target !== undefined &&
                    (options.target === _this.locals.formName || options.target === _this.locals.target)) {
                    /**
                                         * this code is clear, it's used to pass actions from a form element to the parent form
                                         * that will forword those actions along with the form values
                                         */
                    if (options.formaction) {
                        /* the usual, make a immutable clone to prevent options being modified */
                        var new_options = cloneDeep(options);
                        _this.DSF.OAA(new_options.locals, { values: _this.emitValues() });
                        new_options.action = new_options.formaction;
                        new_options.target = undefined;
                        new_options.formaction = undefined;
                        _this.DFCS.formChanged(new_options);
                    }
                    else {
                        /**
                                                 * this code is not clear, not sure in what context it is used
                                                 */
                        if (options.values && !isEmpty(options.values)) {
                            _this.patchValues(options.values);
                        }
                        if (options.locals) {
                            _this.DSF.OAA(_this.locals, options.locals);
                        }
                        if (options.formSpecs) {
                            _this.DSF.OAA(_this.formSpecs, options.formSpecs);
                        }
                        if (options.formaction) {
                            _this.DSF.OAA(options.locals, { values: _this.emitValues() });
                            options.action = options.formaction;
                            options.target = undefined;
                            options.formaction = undefined;
                            _this.DFCS.formChanged(options);
                        }
                    }
                    _this.check();
                }
            });
        };
        this.fromServer = function () {
            _this.DSF.loadForm(_this.DSF.OAA(_this.values, _this.locals.params), _this.locals, _this.storeName);
        };
        this.setDefaultValues = function () {
            var vals;
            if (_this.locals.input) {
                var obj_1 = _this.emitValues();
                vals = reduce(_this.locals.input, function (memo, _value, key) {
                    /* this is diff  */
                    memo[key] = obj_1[key];
                    return memo;
                }, {});
            }
            else {
                vals = _this.values;
            }
            _this.patchValues(vals);
        };
        /**
             *  a function to flatten values
             *
             *
             * @memberOf DFComponent
             */
        this.onSubmit = function (val) {
            _this.DFCS.formChanged({ action: 'submit', values: _this.emitValues() });
            if (val) {
                _this.form.reset(true);
            }
            else {
                _this.formSpecs.readmode = true;
                _this.formSpecs.submitted = true;
            }
        };
        this.onSave = function (val, diff) {
            if (val) {
                _this.formSpecs.readmode = true;
            }
            if (diff) {
                _this.DFCS.formChanged({ action: 'save', values: _this.diff(_this.emitValues(), _this.oldValues) });
            }
            else {
                _this.DFCS.formChanged({ action: 'save', values: _this.emitValues() });
            }
        };
        this.onEdit = function () {
            _this.oldValues = cloneDeep(_this.form.value);
            _this.oldquestionGroups = cloneDeep(_this.questionGroups);
            _this.formSpecs.readmode = false;
            _this.DFCS.formChanged({ action: 'edit', values: _this.emitValues() });
        };
        this.onCancel = function () {
            if (_this.oldValues) {
                _this.form.patchValue(_this.oldValues);
                _this.questionGroups = _this.oldquestionGroups;
            }
            _this.formSpecs.readmode = true;
            _this.DFCS.formChanged({ action: 'cancel', values: _this.emitValues() });
        };
        this.onDelete = function () {
            _this.form.reset(true);
            _this.formSpecs.readmode = true;
            _this.DFCS.formChanged({ action: 'delete', values: _this.emitValues() });
        };
        /**
             *  this code is triggered externallt by evaluating an onclick on a form button
             * action
             * @memberOf DFComponent
             */
        this.onAction = function (action) {
            var vals;
            if (_this.locals.output) {
                var obj_2 = _this.emitValues();
                /* tslint:disable:no-identical-functions */
                vals = reduce(_this.locals.output, function (memo, _value, key) {
                    /* this is diff  */
                    memo[key] = obj_2[key];
                    return memo;
                }, {});
            }
            else {
                vals = _this.emitValues();
            }
            _this.formSpecs.spinner = true;
            _this.DFCS.formChanged({ action: action, values: vals, params: _this.locals.params || undefined });
        };
        this.onValueChanged = function () {
            if (_this.form) {
                _this.formErrors = _this.qcs.getFormErrors(_this.form, _this.questions, _this.validations, _this.componentName, _this.formSpecs.messages);
            }
            return;
        };
        this.evalStr = function (str) {
            return eval(str);
        };
        this.questionsVisible = function (questions) {
            for (var _i = 0, questions_1 = questions; _i < questions_1.length; _i++) {
                var question = questions_1[_i];
                if (!question.hidden) {
                    return true;
                }
            }
            return false;
        };
        this.patchValues = function (values) {
            if (isEmpty(values)) {
                return;
            }
            // this.form.patchValue(values);
            var o = {};
            for (var v in values) {
                if (_this.form.controls.hasOwnProperty(v)) {
                    var q = _this.qcs.getQuestion(v, _this.questions);
                    if (q && q.value !== values[v]) {
                        q.value = values[v];
                    }
                    if (_this.form.controls[v].value !== values[v]) {
                        o[v] = values[v];
                    }
                }
            }
            if (!isEmpty(o)) {
                _this.form.patchValue(o, { emitEvent: true });
                _this.check();
            }
        };
        this.findIdx = function (arr, field, value) {
            var i = findIndex(arr, function (el) {
                return el[field] === value;
            });
            return i;
        };
        this.setPath = function (path, value, notation, values) {
            var isObject = function (obj) {
                return (Object.prototype.toString.call(obj) === '[object Object]' && !!obj);
            };
            notation = notation || '.';
            var t = path.split(notation).reduce(function (prev, cur, idx, arr) {
                var isLast = (idx === arr.length - 1);
                if (isLast) {
                    return (prev[cur] = value);
                }
                // if <cur> is not last part of path, then returns object if existing value is object or empty object
                return (isObject(prev[cur])) ? prev[cur] : (prev[cur] = {});
            }, values);
            return t;
        };
        this.diff = function (obj1, obj2) {
            if (obj2 === undefined) {
                return obj1;
            }
            var r = {};
            for (var prop in obj1) {
                if (obj1.hasOwnProperty(prop)) {
                    if (obj1[prop] !== obj2[prop]) {
                        r[prop] = obj1[prop];
                    }
                }
            }
            return r;
        };
        this.check = function () {
            _this.cd.markForCheck();
        };
        this.env = env;
        this.cd = cd;
        this.fs = fs;
        this.qcs = qcs;
        this.store = store;
        this.DSF = DSF;
        this.DFCS = DFCS;
        this.loaded = false;
        this.storeValues = {};
    }
    DFComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.setForm();
        /** get the real name of the store (adds a url path) */
        this.storeName = this.DSF.r_getStore(this.locals.store);
        /**
                 * if the config locals contains the reset flag , then reset all values
                 * values are normally retained in the questions object
                 */
        if (this.locals && this.locals.reset) {
            this.resetQuestions();
            this.DSF.resetFormStore(this.storeName);
        }
        if (this.locals && this.locals.values && !isEmpty(this.locals.values)) {
            this.patchValues(this.locals.values);
        }
        /** Listen to any incomming messages from other forms
                 * and update the values
                 */
        this.formSubscribe();
        /** wait for any results
                 * this is a general listerner without any flag
                 */
        this.awaitResults();
        /** if data are required from any store , then get them here */
        if (this.locals.required) {
            this.requiredFromStore();
        }
        /** if the data are to be found in the store, then get them from that store */
        if (this.locals.fromStore) {
            if (Array.isArray(this.locals.store)) {
                each(this.locals.store, function (store) {
                    var storeName = _this.DSF.r_getStore(store);
                    _this.fromStore(storeName);
                });
            }
            else {
                this.fromStore(this.storeName);
            }
        }
        /** if the data are to be found in the store, then get them from that store */
        if (this.locals.fromServer) {
            this.fromServer();
        }
        /** listen to changes */
        this.watchChanges();
        /** refresh  */
        // this.check();
        console.info("$df.form.component (ngOnInit) => " + this.locals.formName);
    };
    DFComponent.prototype.setForm = function () {
        var _this = this;
        this.form = this.qcs.toFormGroup(this.questionGroups, this.formSpecs.disabled, this.validations, this.questions, this.keyNames, this.formSpecs.messages);
        if (this.values) {
            this.setDefaultValues();
        }
        each(this.questions, function (q) {
            if (q.key) {
                _this.form.value[q.key] = q.value;
            }
        });
        this.oldValues = cloneDeep(this.form.value);
        this.onValueChanged();
        this.check();
    };
    /**
     *
     * here we subscribe to the store and await results
     * @returns {*}
     *
     * @memberOf DFComponent
     */
    /**
         *
         * here we subscribe to the store and await results
         * @returns {*}
         *
         * @memberOf DFComponent
         */
    DFComponent.prototype.awaitResults = /**
         *
         * here we subscribe to the store and await results
         * @returns {*}
         *
         * @memberOf DFComponent
         */
    function () {
        var _this = this;
        if (!this.locals.filterStore) {
            return;
        }
        var store = this.DSF.r_getStore(this.locals.filterStore);
        this.AStore = this.store
            .select(function (s) { return s.coverage.states[store]; })
            .subscribe(function (data) {
            if (data !== undefined) {
                /** in case there is a spinner
                                     * the spinner is used in the button to disable or enable it
                                     * but could be used for any other purpose when data are updated
                                     * from the store
                                     */
                /** in case there is a spinner
                 * the spinner is used in the button to disable or enable it
                 * but could be used for any other purpose when data are updated
                 * from the store
                 */
                _this.formSpecs.spinner = false;
                _this.check();
            }
        });
        return;
    };
    /**
     * here we load data from a store if the fromStore is set
     * We do this only if the formname is different from the local name to avoid getting
     * into a loop because of the value changes;
     * @returns {*}
     *
     * @memberOf DFComponent
     */
    /**
         * here we load data from a store if the fromStore is set
         * We do this only if the formname is different from the local name to avoid getting
         * into a loop because of the value changes;
         * @returns {*}
         *
         * @memberOf DFComponent
         */
    DFComponent.prototype.fromStore = /**
         * here we load data from a store if the fromStore is set
         * We do this only if the formname is different from the local name to avoid getting
         * into a loop because of the value changes;
         * @returns {*}
         *
         * @memberOf DFComponent
         */
    function (store) {
        var _this = this;
        this.FStore = this.store
            .select(function (s) { return s.coverage.states[store]; })
            .subscribe(function (data) {
            if (data !== undefined
                && !isEmpty(data.values)
                && !isEqual(_this.storeValues, data)) {
                /** date from the server have a loaded flag, we set the component loaded flag to true */
                if (data.loaded) {
                    _this.loaded = true;
                }
                /** in case there is a spinner
                                     * the spinner is used in the button to disable or enable it
                                     * but could be used for any other purpose when data are updated
                                     * from the store
                                     */
                /** in case there is a spinner
                 * the spinner is used in the button to disable or enable it
                 * but could be used for any other purpose when data are updated
                 * from the store
                 */
                _this.formSpecs.spinner = false;
                /** to avoid the form to update itself with the onchanges listerner we set a block to true */
                /** make the values immutable */
                /** to avoid the form to update itself with the onchanges listerner we set a block to true */
                /** make the values immutable */
                _this.storeValues = cloneDeep(data);
                _this.oldValues = cloneDeep(_this.form.value);
                var values_1 = cloneDeep(data.values);
                /**
                                     * data can be loaded from the store but also from the form itself that dispatches
                                     * the fromStore can be a boolean but also an object
                                     * we can use this to only let values in from a specific store
                                     * that means our store can be updated without the front end store being updated
                                     */
                if (!isBoolean(_this.locals.fromStore)) {
                    var storeName = _this.DSF.r_getStore(_this.locals.fromStore);
                    if (data.target.indexOf(storeName) === -1) {
                        return;
                    }
                }
                /** allow only spefic values */
                if (_this.locals.input) {
                    var flatValues_1 = _this.DSF.flatten(values_1);
                    values_1 = reduce(_this.locals.input, function (memo, _value, key) {
                        var t = memo;
                        t[key] = flatValues_1[key];
                        return t;
                    }, {});
                }
                /**
                                     * This code is used to set a field value from a different based on the fact if another form is valid or not
                                     */
                if (values_1 && data.form.name && data.form.name !== '') {
                    // only if we have values
                    /** find out of there's a filed that has a key with name of the form to check */
                    var i = _this.findIdx(_this.questions, 'label', data.form.name);
                    if (i > -1) {
                        // the form needs to have a name
                        var key = _this.questions[i].key;
                        values_1[key] = data.form.valid
                            ? data.form.valid
                            : '';
                    }
                    else {
                        var t = _this.findIdx(_this.questions, 'label', 'form');
                        if (t > -1) {
                            // the form needs to have a name
                            _this.questions[t].label = data.form.name;
                            var key = _this.questions[t].key;
                            values_1[key] = data.form.valid
                                ? data.form.valid
                                : '';
                        }
                    }
                }
                if (_this.locals.map) {
                    var pvalues_1 = cloneDeep(values_1);
                    each(_this.locals.map.fields, function (f) {
                        if (f.date && values_1[f.in]) {
                            values_1[f.in] = new Date(pvalues_1[f.out]);
                        }
                        else {
                            _this.setPath(f.in, pvalues_1[f.out], undefined, values_1);
                        }
                        // values[f.in] = pvalues[f.out];
                    });
                }
                _this.patchValues(values_1);
                // this.check();
            }
        });
        return;
    };
    DFComponent.prototype.requiredFromStore = function () {
        var _this = this;
        var required = this.locals.required;
        this.RStore = this.store
            .select(function (s) { return s.coverage.states[_this.DSF.r_getStore(required.store)]; })
            .take(1)
            .subscribe(function (data) {
            /** we need to have data AND the data are not already loaded from the sever */
            if (data !== undefined && !isEmpty(data.values)) {
                var values_2 = {};
                if (required.fields) {
                    var pvalues_2 = cloneDeep(data.values);
                    if (required.flatten) {
                        pvalues_2 = _this.DSF.flatten(pvalues_2);
                    }
                    each(required.fields, function (f) {
                        if (f.date) {
                            values_2[f.in] = new Date(pvalues_2[f.out]);
                        }
                        else {
                            _this.setPath(f.in, pvalues_2[f.out], undefined, values_2);
                        }
                        // values[f.in] = pvalues[f.out];
                    });
                }
                else {
                    values_2 = data.values;
                }
                /*
                                    if (this.locals.required.displayField) {
                                        values = { [this.locals.required.displayField]: values };
                                    }
                                    */
                /*
                if (this.locals.required.displayField) {
                    values = { [this.locals.required.displayField]: values };
                }
                */
                _this.patchValues(values_2);
                _this.check();
                /** if there's an url in the locals it means we have now a key from the store ad can load the form */
                if (_this.locals.url && !_this.loaded) {
                    _this.DSF.loadForm({
                        query: values_2,
                    }, _this.locals, _this.storeName);
                }
            }
        });
        return;
    };
    DFComponent.prototype.watchChanges = function () {
        var _this = this;
        this.VStore = this.form.valueChanges
            .subscribe(function (data) {
            if (data !== undefined) {
                _this.onValueChanged();
            }
            /** if data are to be loaded into the store then enable this option */
            if (_this.locals.toStore) {
                if (isEqual(_this.oldValues, data)) {
                    return;
                }
                _this.oldValues = cloneDeep(_this.form.value);
                _this.oldquestionGroups = cloneDeep(_this.questionGroups);
                var nvalues = void 0;
                if (_this.locals.output) {
                    nvalues = reduce(_this.locals.output, function (memo, _value, key) {
                        memo[key] = _this.form.value[key];
                        return memo;
                    }, {});
                }
                else {
                    nvalues = _this.form.value;
                }
                var values = _this.DSF.OAA(__assign({}, _this.storeValues.values), nvalues);
                var form = {
                    error: false,
                    form: {
                        name: _this.locals.formName,
                        valid: _this.form.valid,
                    },
                    loaded: true,
                    store: _this.storeName,
                    target: [_this.storeName],
                    values: values,
                };
                /** create a block so that the component will not update itself from the from store
                                     */
                if (isEqual(_this.storeValues, form)) {
                    return;
                }
                _this.patchValues(values);
                _this.storeValues = cloneDeep(form);
                _this.DSF.updateForm(_this.storeName, form);
            }
        });
    };
    DFComponent.prototype.resetQuestions = function () {
        var keys1 = this.keyNames;
        for (var _i = 0, keys1_1 = keys1; _i < keys1_1.length; _i++) {
            var key = keys1_1[_i];
            var question = this.qcs.getQuestion(key, this.questions);
            question.value = undefined;
        }
    };
    DFComponent.prototype.getStoreSubscription = function () {
        var _this = this;
        var store = this.locals.filterStore
            ? this.DSF.r_getStore(this.locals.filterStore)
            : this.storeName;
        var subscription;
        this.FSStore = this.store
            .select(function (s) { return s.coverage.states[store]; })
            .take(1) // only subscribe once
            .subscribe(function (data) {
            subscription = {
                data: data,
                store: _this.DSF.r_getStore(_this.locals.store),
            };
        });
        return subscription;
    };
    DFComponent.prototype.onFilter = function (reset, keys) {
        /* tslint:disable:no-parameter-reassignment */
        keys = !keys
            ? this.keyNames
            : keys;
        var filterValues = cloneDeep(this.getStoreSubscription());
        if (filterValues && filterValues.data && filterValues.data.query) {
            var query = filterValues.data.query;
            for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                var key = keys_1[_i];
                if (this.form.controls.hasOwnProperty(key)) {
                    var t = this.form.controls[key];
                    query[key] = t.value && t.value !== ''
                        ? t.value
                        : undefined;
                }
                if (reset) {
                    query[key] = undefined;
                    /* reset the values and also set readmodes */
                    var q = this.qcs.getQuestion(key, this.questions);
                    if (q) {
                        q.value = undefined;
                        if (q.readOnly) {
                            q.readOnly = false;
                        }
                    }
                }
            }
            /* the reset should reset the form and not reload the data */
            if (reset) {
                this.form.reset(this.values);
            }
            else {
                var store = this.locals.filterStore
                    ? this.DSF.r_getStore(this.locals.filterStore)
                    : filterValues.store;
                this.formSpecs.spinner = true;
                this.fs.updateFilter({
                    query: query,
                    store: store,
                });
            }
        }
    };
    DFComponent.prototype.getAction = function (action) {
        if (action.action) {
            var values = this.emitValues();
            if (action.values) {
                this.DSF.OAA(values, action.values);
            }
            action.values = values;
            this.DFCS.formChanged(action);
            action.values = {}; // reset these action values to prevent them to persist future changes;
            return;
        }
        this.evalStr(action);
    };
    DFComponent.prototype.ngOnDestroy = function () {
        if (this.formSubscription) {
            this.formSubscription.unsubscribe();
        }
        if (this.FSStore) {
            this.FSStore.unsubscribe();
        }
        if (this.AStore) {
            this.AStore.unsubscribe();
        }
        if (this.RStore) {
            this.RStore.unsubscribe();
        }
        if (this.FStore) {
            this.FStore.unsubscribe();
        }
        if (this.VStore) {
            this.VStore.unsubscribe();
        }
        if (this.TStore) {
            this.TStore.unsubscribe();
        }
        console.info("$df.form.component (ngOnDestroy) => destroying " + this.locals.formName);
    };
    /** a function that add to the original values the values from the form
     * this.form values has the highest level;
     */
    /** a function that add to the original values the values from the form
         * this.form values has the highest level;
         */
    DFComponent.prototype.emitValues = /** a function that add to the original values the values from the form
         * this.form values has the highest level;
         */
    function () {
        var val = this.form.value;
        if (this.values) {
            val = this.DSF.OA(this.values, val);
        }
        if (this.locals.values) {
            val = this.DSF.OA(this.locals.values, val);
        }
        return val;
    };
    DFComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [DFQuestionControlService],
                    selector: 'df-form',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFComponent.ctorParameters = function () { return [
        { type: Env, },
        { type: ChangeDetectorRef, },
        { type: DFFilterService, },
        { type: DFQuestionControlService, },
        { type: Store, },
        { type: DFStoreFunctions, },
        { type: DFCommonService, },
    ]; };
    DFComponent.propDecorators = {
        "questionGroups": [{ type: Input },],
        "values": [{ type: Input },],
        "componentName": [{ type: Input },],
        "locals": [{ type: Input },],
        "formSpecs": [{ type: Input },],
    };
    return DFComponent;
}());
export { DFComponent };
