'use strict';
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { DFFilterService } from '../services/store/df.filter.service';
import { DFStoreFunctions } from '../services/store/df.store.functions';
import { Store } from '@ngrx/store';
import 'rxjs/add/operator/takeWhile';
import { tmpl } from './templates/df.list.pug';
import { DFCommonService } from '../services/df.common.api';
import { default as each } from 'lodash-es/each';
import { default as isEmpty } from 'lodash-es/isEmpty';
import { default as cloneDeep } from 'lodash-es/cloneDeep';
var DFListComponent = /** @class */ (function () {
    function DFListComponent(store, DSF, route, cd, FS, DFCS) {
        var _this = this;
        this.store = store;
        this.DSF = DSF;
        this.route = route;
        this.cd = cd;
        this.FS = FS;
        this.DFCS = DFCS;
        this.block = false;
        this.formSubscribe = function () {
            _this.formSubscription = _this.DFCS.form$
                .subscribe(function (options) {
                if (options.target && (options.target === _this.locals.formName || options.target === _this.locals.target)) {
                    switch (options.action) {
                        case 'reload':
                            _this.loadIt(event);
                            break;
                        case 'default':
                            return;
                    }
                }
            });
        };
        this.onRowSelect = function (event) {
            _this.DFCS.formChanged({ action: 'selection', target: _this.locals.target, values: event.data });
        };
        this.concatMulti = function (field, concat) {
            if (typeof field === 'string') {
                return field;
            }
            else if (Array.isArray(field)) {
                var obj_1 = [];
                field.forEach(function (el) {
                    obj_1.push(el[concat] || el[0]);
                });
                return obj_1.join(', ');
            }
            else {
                return field[concat];
            }
        };
        this.getValue = function (item, field, subfield) {
            if (subfield === undefined) {
                return item[field];
            }
            return item[field][subfield];
        };
        this.rowClass = function (data) {
            if (_this.formSpecs.rowClassField) {
                return data[_this.formSpecs.rowClassField];
            }
            return;
        };
        this.checknode = function (evalP) {
            if (evalP instanceof HTMLCollection || evalP instanceof NodeList || evalP instanceof HTMLElement) {
                return true;
            }
            return false;
        };
        this.check = function () {
            _this.cd.markForCheck();
        };
        this.loaded = false;
        this.spinner = false;
    }
    DFListComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.loading = this.formSpecs.lazy ? true : false; // if there's lazy load then no default loading
        this.links = this.formSpecs.links || [];
        this.icons = this.formSpecs.icons || [];
        this.table = this.formSpecs.table;
        /* set the params to a query object in case it's undefined */
        if (this.params === undefined) {
            this.params = { query: {} };
        }
        /** make sure we have a value for the paginator */
        this.formSpecs.paginator = this.formSpecs.paginator === false ? false : true;
        /** detailstype is used in the table for the expander links , text etx..
                 * yçu can even embed a form using formLocals
                 */
        if (this.formSpecs.details) {
            this.detailsType = this.formSpecs.details.type;
            if (this.formSpecs.details.formName !== undefined) {
                if (this.formLocals === undefined) {
                    this.formLocals = {
                        formName: this.formSpecs.details.formName,
                    };
                }
                else {
                    this.formLocals.formName = this.formSpecs.details.formName;
                }
            }
        }
        else {
            this.formSpecs.details = {};
        }
        this.evaluate(this.locals);
        this.evaluate(this.params);
        /** In case store data are loaded from the component, no need to load it from server */
        if (this.storeData !== undefined) {
            console.info('$df.list.compnent (ngOnit) => storeData Provided');
            return;
        }
        /* get the storeName */
        this.storeName = this.locals.nested
            ? this.DSF.r_getStore(this.locals.store + "-" + this.values[this.locals.nested])
            : this.DSF.r_getStore(this.locals.store);
        console.info("$df.list.component (ngOnit) => store = " + this.storeName);
        // Initializing the store with default values, if already exists, then store will be reused
        this.DSF.initStore(this.storeName, this.params.query);
        // listen for messages
        this.formSubscribe();
        if (this.locals.reset) {
            /** ia rest has been asked so we are resetting the store */
            this.DSF.resetStore(this.storeName);
        }
        // Subscribing to that store
        this.FSStore = this.store
            .select(function (s) { return s.coverage.states[_this.storeName]; })
            .subscribe(function (data) {
            _this.storeData = cloneDeep(data);
            if (_this.storeData.query) {
                if (_this.params && _this.params.query) {
                    _this.params.query = _this.DSF.OA(_this.params.query, _this.storeData.query);
                }
            }
            if (_this.formSpecs.preload) {
                _this.formSpecs.styleClass = 'table-loaded';
                _this.spinner = false;
            }
            /* a class to indicate table is loaded */
            if (_this.storeData.loaded) {
                if (_this.loaded) {
                    _this.locals.loaded = false;
                    _this.formSpecs.lazy = true;
                }
                else {
                    _this.locals.loaded = true;
                    if (_this.formSpecs.lazy) {
                        _this.formSpecs.lazy = false;
                        setTimeout(function () {
                            _this.locals.loaded = false;
                            _this.formSpecs.lazy = true;
                        }, 0);
                    }
                }
                _this.formSpecs.styleClass = 'table-loaded';
                _this.spinner = false;
                _this.DFCS.formChanged({ table: _this.formSpecs.id, loaded: true });
            }
            _this.check();
        });
        this.FSSub = this.FS.filterUpdate.subscribe(function (filter) {
            if (filter.store === _this.storeName) {
                /** get the params from the filter and apply them to the table */
                /** get the params from the filter and apply them to the table */
                _this.DSF.OAA(_this.params.query, filter.query);
                _this.loading = true;
                _this.locals.loaded = false;
                _this.formSpecs.lazy = true;
                _this.loadIt({}); // reset any filtes
            }
        });
        /* If we have required query parameters on the params then
                * we need to assume that they need data from the store
                * so lets get them for the store
                */
        if (this.formSpecs.type.indexOf('tree') > -1) {
            this.loadIt({});
        }
        // Read queryparams from the url and if found object merge them to the params.query
        this.params.query = this.DSF.OA(this.params.query, this.DSF.r_qP(this.route));
    };
    DFListComponent.prototype.Lazyload = function (event) {
        var _this = this;
        /** no lazy loading when parameter lazy is set to false  */
        if (!this.formSpecs.lazy) {
            this.check();
        }
        else if (this.locals.required) {
            var required_1 = this.locals.required;
            this.block = true; // we need to wait this the store is loaded , we will connect to the store and once it's filled up
            var storeId_1 = this.DSF.r_getStore(required_1.store);
            this.FFSSub = this.store
                .select(function (s) { return s.coverage.states[storeId_1]; })
                .takeWhile(function () { return _this.block; }) // we listen until block is false;
                .subscribe(function (data) {
                /** we need to have data
                                     * AND data values
                                     * and ( also the data are not already loaded from the sever
                                     * or they may have been loaded but a reset has been asked
                                     * and ( the number of rows are different or the first are different)
                                     */
                if (/*
                                        data &&
                                        !isEmpty(data.values) &&
                                        (
                                            (!data.loaded || data.loaded && this.locals.reset) ||
                                            (this.params.query.rows !== event.rows || this.params.query.first !== event.first)
                                        )
                                        */
                data && !isEmpty(data.values)) {
                    var obj_2 = {};
                    each(required_1.fields, function (f) {
                        obj_2[f.in] = data.values[f.out];
                    });
                    _this.params.query = _this.DSF.OA(_this.params.query, obj_2);
                    _this.block = false;
                    /* wehn the components may only be initialized if data it depends on has been loaded
                                             * if it has not been loaded then the component is initialized without data
                                             * therefore we need to load the data once the dependend data come in .
                                             */
                    /* wehn the components may only be initialized if data it depends on has been loaded
                     * if it has not been loaded then the component is initialized without data
                     * therefore we need to load the data once the dependend data come in .
                     */
                    _this.loadIt(event);
                }
            });
        }
        else {
            this.loadIt(event);
        }
    };
    DFListComponent.prototype.loadIt = function (event) {
        /* as soon as the table gets on the screen, it starts to call the lazy load
                * we check if there is already a store
                * that the store name is the same
                * and that data are loaded
                */
        if (this.block) {
            this.check();
            return; // we DON'T load anyting as we need to load from store, we should have the id above !!!
        }
        /** the reason why !locals.loaded is that a new component alsways starts with loaded false
                 * so that if store data is loaded but locals.loaded is false then we know it's a new component that needs
                 * to use existing data
                 */
        if (this.storeData.loaded && this.locals.loaded && this.storeData.store === this.storeName) {
            this.locals.loaded = true;
            this.check();
            return console.info("$df.list.component (Lazyload) => using existing store : " + this.storeName);
        }
        this.locals.loaded = true;
        this.loaded = true;
        /**
                 * merge the objects, we want to have the complete search object but it should also work
                 * with pagination (event.rows)
                 * from left to right api first and rows are checked
                 */
        this.params = Object.assign({}, this.params, {
            query: Object.assign({}, this.params.query, {
                api: this.locals.api !== undefined
                    ? this.locals.api
                    : this.params.query.api !== undefined
                        ? this.params.query.api
                        : undefined,
                first: event.first !== undefined
                    ? event.first
                    : this.params.query.first !== undefined
                        ? this.params.query.first
                        : 0,
                rows: event.rows !== undefined
                    ? event.rows
                    : this.params.query.rows !== undefined
                        ? this.params.query.rows
                        : 50,
                sortField: event.sortField !== undefined
                    ? event.sortField
                    : this.params.query.sortField !== undefined
                        ? this.params.query.sortField
                        : undefined,
                sortOrder: event.sortOrder !== undefined
                    ? event.sortOrder
                    : this.params.query.sortOrder !== undefined
                        ? this.params.query.sortOrder
                        : undefined,
            }),
            url: this.locals.url !== '' ? this.locals.url : this.locals.proxyurl,
        });
        this.load();
    };
    DFListComponent.prototype.load = function () {
        this.spinner = true;
        this.formSpecs.styleClass = 'table-loading';
        this.DSF.loadStore(this.params, this.locals, this.storeName);
        this.check();
    };
    DFListComponent.prototype.ngOnDestroy = function () {
        /** a security to ensure that we always start with a not loaded component
                 * usecase when the localdata is not destroyed
                 */
        this.locals.loaded = false;
        /** prevent memory leak when component destroyed */
        if (this.FSSub !== undefined) {
            this.FSSub.unsubscribe();
        }
        if (this.FSStore !== undefined) {
            this.FSStore.unsubscribe();
        }
        if (this.FFSSub !== undefined) {
            this.FFSSub.unsubscribe();
        }
        if (this.formSubscription !== undefined) {
            this.formSubscription.unsubscribe();
        }
        console.info("$df.list.component (ngOnDestroy) => destroying store " + this.storeName);
    };
    DFListComponent.prototype.onRowExpand = function (_event) {
        this.check();
        return;
    };
    DFListComponent.prototype.isType = function (_type) {
        if (this.detailsType && this.detailsType.length > 0) {
            return true;
        }
        return false;
    };
    DFListComponent.prototype.getResult = function (result) {
        this.DFCS.formChanged(result);
    };
    DFListComponent.prototype.loadNode = function (event) {
        if (event.node) {
            var paramsQuery = [];
            var pVal = void 0;
            for (var _i = 0, _a = this.formSpecs.query; _i < _a.length; _i++) {
                var q = _a[_i];
                if (q.value instanceof Array) {
                    for (var _b = 0, _c = q.value; _b < _c.length; _b++) {
                        var v = _c[_b];
                        pVal = !pVal ? event.node.data[v] : pVal + "," + event.node.data[v];
                    }
                }
                else {
                    pVal = event.node.data[q.value];
                }
                paramsQuery[q.key] = pVal;
            }
            this.params.query = paramsQuery;
            if (!event.node.children) {
                this.DSF.loadNode(this.params, this.locals, this.storeName, event.node);
            }
        }
    };
    DFListComponent.prototype.evaluate = function (obj) {
        for (var p in obj) {
            if (obj.hasOwnProperty(p)) {
                var evalP = obj[p];
                if (evalP !== '') {
                    try {
                        evalP = eval(obj[p]);
                    }
                    catch (e) {
                        evalP = obj[p];
                    }
                    if (obj[p] !== evalP) {
                        if (!this.checknode(evalP)) {
                            obj[p] = evalP;
                        }
                    }
                    if (typeof obj[p] === 'object') {
                        this.evaluate(obj[p]);
                    }
                }
            }
        }
    };
    DFListComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    selector: 'df-list',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFListComponent.ctorParameters = function () { return [
        { type: Store, },
        { type: DFStoreFunctions, },
        { type: ActivatedRoute, },
        { type: ChangeDetectorRef, },
        { type: DFFilterService, },
        { type: DFCommonService, },
    ]; };
    DFListComponent.propDecorators = {
        "locals": [{ type: Input },],
        "params": [{ type: Input },],
        "storeData": [{ type: Input },],
        "query": [{ type: Input },],
        "formSpecs": [{ type: Input },],
        "formLocals": [{ type: Input },],
        "values": [{ type: Input },],
    };
    return DFListComponent;
}());
export { DFListComponent };
