'use strict';
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, HostListener, Input, } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { DFQuestionControlService } from '../services/df.question.control.service';
import { DFFormService } from '../services/df.form.service';
import { HttpService } from 'io-ng-util/lib/common/http.service';
import { tmpl } from './templates/df.question.pug';
import { DFCommonService } from '../services/df.common.api';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/distinctUntilChanged';
import { Env } from 'io-ng-util/lib/common/env';
import { default as mapValues } from 'lodash-es/mapValues';
var DFQuestionComponent = /** @class */ (function () {
    function DFQuestionComponent(env, DFCS, cd, formService, httpService, qcs) {
        var _this = this;
        this.env = env;
        this.DFCS = DFCS;
        this.cd = cd;
        this.formService = formService;
        this.httpService = httpService;
        this.qcs = qcs;
        this.dependents = [];
        this.fieldsToWatch = [];
        this.depsToWatch = [];
        this.permVisible = false;
        this.permEditable = false;
        this.uploadedFiles = [];
        this.onAction = function (action) {
            _this.DFCS.formChanged(action);
            return false;
        };
        this.onSelect = function (e) {
            if (_this.question.flatValues) {
                _this.form.patchValue(e);
            }
            if (_this.question.actions) {
                _this.DFCS.formChanged({
                    action: _this.question.actions.action,
                    params: _this.question.actions.params,
                    values: (_a = {}, _a[_this.question.key] = e, _a),
                });
            }
            _this.question.value = e;
            _this.question.readOnly = true;
            _this.question.suggestions = undefined;
            var _a;
        };
        this.onReset = function () {
            if (_this.question.flatValues) {
                var values = mapValues(_this.question.value, function () { return undefined; });
                _this.form.patchValue(values);
            }
            var q = _this.form.get(_this.question.key);
            if (q) {
                q.setValue(undefined);
            }
            _this.question.value = undefined;
            _this.question.readOnly = false;
            /* if there are some additional actions, then emit them) */
            if (_this.question.resetActions) {
                _this.onAction(_this.question.resetActions);
            }
        };
        /**
             * @opt is the option of the question checked
             * the question is patched
             * the form question is patched
             * @memberof DFQuestionComponent
             */
        this.checkOnChange = function (opt, e) {
            if (e) {
                if (_this.question.value === undefined) {
                    _this.question.value = [];
                }
                _this.question.value.push(opt.value);
            }
            else {
                var idx = _this.question.value.indexOf(opt.value);
                _this.question.value.splice(idx, 1);
            }
            var q = _this.form.get(_this.question.key);
            if (q) {
                q.setValue(_this.question.value);
            }
        };
        this.dragenter = function (e) {
            e.stopPropagation();
            e.preventDefault();
            _this.dropClass = 'dropClass';
            _this.dragTarget = e.target;
            return false;
        };
        this.dragover = function (e) {
            e.stopPropagation();
            e.preventDefault();
            return false;
            // this.dropClass = 'dropClass';
        };
        this.dragleave = function (e) {
            if (e.target === _this.dragTarget) {
                e.stopPropagation();
                e.preventDefault();
                _this.dropClass = '';
            }
        };
        this.drop = function (e) {
            e.stopPropagation();
            e.preventDefault();
            if (!_this.question.multiple) {
                _this.uploadedFiles = [];
            }
            var files = e.dataTransfer ? e.dataTransfer.files : e.target.files;
            _this.addFiles(files);
            var q = _this.form.get(_this.question.targetQuestion);
            if (q) {
                q.setValue(_this.uploadedFiles);
            }
            _this.cd.detectChanges();
            _this.cd.markForCheck();
            _this.dropClass = '';
        };
        this.addFiles = function (files) {
            var _loop_1 = function (file) {
                if ((_this.question.maxLength === undefined || false) ||
                    (_this.question.maxLength && (file.size / 1024 / 1024 < _this.question.maxLength))) {
                    var idx = _this.uploadedFiles.findIndex(function (x) { return (x.name === file.name && x.size === file.size); });
                    if (idx === -1) {
                        _this.uploadedFiles.push(file);
                    }
                }
            };
            for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
                var file = files_1[_i];
                _loop_1(file);
            }
        };
        /*
            private watchFields(): void {
                this.fieldsToWatch.forEach((fieldW) => {
                    this.form
                        .get(fieldW).valueChanges
                        .debounceTime(200)
                        .subscribe((data) => {
                            if (
                                ['text', 'checkbox'].indexOf(this.question.controlType) === -1) { // not in
                                this.question.value = data;
                            }
                        });
                });
            }
            */
        this.evalStr = function (str) {
            return eval(str);
        };
        this.check = function () {
            _this.cd.markForCheck();
        };
    }
    Object.defineProperty(DFQuestionComponent.prototype, "isValid", {
        get: function () { return this.form.controls[this.question.key].valid; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DFQuestionComponent.prototype, "isVisible", {
        get: function () {
            var x = true;
            if (this.question.hideMode && this.question.hideMode === 'edit') {
                x = false;
            }
            if (this.question.hidden) {
                x = false;
            }
            return x;
        },
        enumerable: true,
        configurable: true
    });
    DFQuestionComponent.prototype.ngOnInit = function () {
        /*
                if (this.question.key) {
                    this.fieldsToWatch.push(this.question.key);
                    this.watchFields();
                }
                */
        if (this.question.suggestions) {
            this.question.suggestions = [];
        }
        if (this.question.fromEnv) {
            var v = this.question.fromEnv;
            var t = this.env;
            this.question.value = (t[v.key][v.value]);
            var q = this.form.get(this.question.key);
            if (q) {
                q.setValue(this.question.value);
            }
        }
        if (this.question.dependency) {
            this.depsToWatch = this.qcs.getDependents(this.question, this.fieldsToWatch, this.dependents, this.formQuestions);
            this.watchDependents();
            this.onValueChanged(this.question.value); // initital check
        }
        /** do an iniital refresh  */
        this.check();
    };
    DFQuestionComponent.prototype.search = function (event) {
        var _this = this;
        if (this.question.jsonp) {
            var url = this.question.urlPath + event.query + (this.question.jsonp ? '&callback=JSONP_CALLBACK&' : '');
            this.httpService.jsonpGet(url)
                .debounceTime(400)
                .distinctUntilChanged()
                .subscribe(function (options) {
                _this.question.suggestions = options;
                _this.check();
            });
        }
        else {
            var url = this.question.urlPath + event.query;
            this.httpService.httpGet(url)
                .debounceTime(400)
                .distinctUntilChanged()
                .subscribe(function (options) {
                _this.question.suggestions = options;
                _this.check();
            });
        }
    };
    DFQuestionComponent.prototype.clickout = function (_e) {
        this.check();
    };
    DFQuestionComponent.prototype.onFileUploadClick = function () {
        var _this = this;
        var input = document.createElement('input');
        input.setAttribute('type', 'file');
        if (this.question.multiple) {
            input.multiple = true;
        }
        input.onchange = function (e) {
            if (!_this.question.multiple) {
                _this.uploadedFiles = [];
            }
            var files = e.dataTransfer ? e.dataTransfer.files : e.target.files;
            _this.addFiles(files);
            var q = _this.form.get(_this.question.targetQuestion);
            if (q) {
                q.setValue(_this.uploadedFiles);
            }
            _this.check();
        };
        input.click();
    };
    DFQuestionComponent.prototype.removeFile = function (file) {
        var idx = this.uploadedFiles.findIndex(function (x) { return (x.name === file.name && x.size === file.size); });
        this.uploadedFiles.splice(idx, 1);
        var q = this.form.get(this.question.key);
        if (q) {
            q.setValue(this.uploadedFiles);
        }
    };
    DFQuestionComponent.prototype.onValueChanged = function (data) {
        var _this = this;
        this.dependents.forEach(function (dep) {
            var field = dep.field;
            switch (dep.type) {
                case 'update-when':
                    var value = _this.evalStr(dep.condition);
                    if (value) {
                        var q0 = _this.form.get(_this.question.key);
                        if (q0) {
                            q0.setValue(undefined);
                        }
                    }
                    break;
                case 'choice-list':
                    field.options = _this.formService.filterOptions(field.optionsInfo, data, field.placeHolder);
                    _this.form.get(field.key).setValue([]);
                    break;
                case 'hide-when':
                    var status_1 = _this.evalStr(dep.condition);
                    _this.question.hidden = status_1;
                    var q1 = _this.form.get(_this.question.key);
                    if (q1) {
                        // we have a question
                        if (status_1) {
                            // condition is true, hide it
                            if (!dep.keepValue) {
                                // there's no flag to keep the value
                                _this.question.value = undefined;
                                q1.setValue(undefined);
                            }
                            if (dep.disable) {
                                // there's a flag to disable the question
                                q1.disable();
                            }
                        }
                        else {
                            if (dep.disable) {
                                // there's a flag to disable the question if condition is true, so enable it
                                q1.enable();
                            }
                        }
                    }
                    break;
                case 'show-when':
                    _this.question.hidden = !_this.evalStr(dep.condition);
                    break;
                case 'disabled-when':
                    var disabled = _this.evalStr(dep.condition);
                    var q = _this.form.get(_this.question.key);
                    if (q) {
                        if (disabled) {
                            q.disable();
                        }
                        else {
                            q.enable();
                        }
                    }
                    break;
                case 'read-only':
                    var readonly = _this.evalStr(dep.condition);
                    _this.question.readOnly = readonly
                        ? true
                        : false;
                    break;
                case 'min-date':
                    _this.question.minDate = _this.evalStr(dep.condition);
                    break;
                case 'max-date':
                    _this.question.maxDate = _this.evalStr(dep.condition);
                    break;
                default:
                    break;
            }
        });
        this.check();
    };
    DFQuestionComponent.prototype.watchDependents = function () {
        var _this = this;
        this.depsToWatch.forEach(function (fieldW) {
            _this.form.controls[fieldW].valueChanges
                .debounceTime(100)
                .subscribe(function (data) { return _this.onValueChanged(data); });
        });
    };
    DFQuestionComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    selector: 'df-question',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFQuestionComponent.ctorParameters = function () { return [
        { type: Env, },
        { type: DFCommonService, },
        { type: ChangeDetectorRef, },
        { type: DFFormService, },
        { type: HttpService, },
        { type: DFQuestionControlService, },
    ]; };
    DFQuestionComponent.propDecorators = {
        "question": [{ type: Input },],
        "formQuestions": [{ type: Input },],
        "form": [{ type: Input },],
        "formErrors": [{ type: Input },],
        "formStyle": [{ type: Input },],
        "disabled": [{ type: Input },],
        "uploadedFiles": [{ type: Input },],
        "clickout": [{ type: HostListener, args: ['document:click', ['$event'],] },],
    };
    return DFQuestionComponent;
}());
export { DFQuestionComponent };
