/*tslint:disable no-forward-ref*/
/*tslint:disable no-use-before-declare*/
'use strict';
import { Component, EventEmitter, forwardRef, Input, Output, } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import 'tinymce';
import 'tinymce/themes/modern';
import 'tinymce/plugins/table';
import 'tinymce/plugins/link';
import 'tinymce/plugins/paste';
import 'tinymce/plugins/image';
import 'tinymce/plugins/contextmenu';
import 'tinymce/plugins/autoresize';
import 'tinymce/plugins/fullscreen';
var EDITOR_VALUE_ACCESSOR = {
    multi: true,
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return TinyEditorComponent; }),
};
var TinyEditorComponent = /** @class */ (function () {
    function TinyEditorComponent() {
        var _this = this;
        // @Output() public onEditorContentChange = new EventEmitter();
        this.uploadClick = new EventEmitter();
        this.tinyDrop = new EventEmitter();
        this.files = [];
        this.editorContent = undefined;
        this.registerOnTouched = function (fn) {
            _this.onTouch = fn;
        };
        this.registerOnChange = function (fn) {
            _this.onModelChange = fn;
        };
        this.writeValue = function (value) {
            _this.editorContent = value;
        };
    }
    TinyEditorComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        tinymce.init({
            branding: false,
            content_style: 'html, body {height: 100%}',
            height: 200,
            menubar: false,
            min_height: 250,
            paste_data_images: true,
            plugins: ['contextmenu', 'fullscreen', 'image', 'link', 'paste', 'table'],
            schema: 'html5',
            selector: "#" + this.elementId,
            setup: function (editor) {
                var self = _this;
                _this.editor = editor;
                editor.on('keyup change', function () {
                    var tinyContent = editor.getContent();
                    _this.onTouch();
                    _this.onModelChange(tinyContent);
                    _this.editorContent = tinyContent;
                });
                var emit = function () {
                    self.uploadClick.emit();
                };
                editor.addButton('clip', {
                    icon: 'clip',
                    onclick: emit,
                    tooltip: 'Upload File',
                });
                editor.on('init', function () {
                    var t = editor.getBody();
                    var stop = function (e) {
                        e.stopPropagation();
                        e.preventDefault();
                    };
                    editor.dom.bind(t, 'dragenter dragover', function (e) {
                        stop(e);
                        return false;
                    });
                    editor.dom.bind(t, 'drop', function (e) {
                        stop(e);
                        _this.tinyDrop.emit(e);
                    });
                });
            },
            skin_url: '../../public/skins/lightgray',
            toolbar: "undo redo | insert | styleselect | bold italic | fullscreen |\n            alignleft aligncenter alignright alignjustify | table | bullist numlist outdent indent | clip",
        });
    };
    TinyEditorComponent.prototype.ngOnDestroy = function () {
        tinymce.remove(this.editor);
    };
    TinyEditorComponent.decorators = [
        { type: Component, args: [{
                    providers: [EDITOR_VALUE_ACCESSOR],
                    selector: 'df-tiny-editor',
                    template: "\n      <textarea id=\"{{elementId}}\"></textarea>\n      <div class='dropZone'><h1 class='dropZoneTitle'>Drop your files here</h1></div>\n  ",
                },] },
    ];
    /** @nocollapse */
    TinyEditorComponent.ctorParameters = function () { return []; };
    TinyEditorComponent.propDecorators = {
        "elementId": [{ type: Input },],
        "uploadClick": [{ type: Output },],
        "tinyDrop": [{ type: Output },],
        "question": [{ type: Input },],
        "files": [{ type: Input },],
    };
    return TinyEditorComponent;
}());
export { TinyEditorComponent };
