'use strict';
import { Injectable } from '@angular/core';
import { HttpService } from 'io-ng-util/lib/common/http.service';
import * as def from '../definitions/index';
import { default as cloneDeep } from 'lodash-es/cloneDeep';
import { default as each } from 'lodash-es/each';
var DFFormService = /** @class */ (function () {
    function DFFormService(http) {
        var _this = this;
        this.http = http;
        this.storedForms = {};
        this.getButtons = function (body) {
            return body.buttons;
        };
        this.getFormStyle = function (body) {
            return body.style;
        };
        this.getPermissions = function (body) {
            return body.roles;
        };
        this.getListForm = function (body) {
            return body.list;
        };
        this.parseGroup = function (groupQuestions) {
            var questions = [];
            each(groupQuestions, function (question) {
                if (question.options) {
                    question.optionsInfo = question.options;
                    question.options = _this.parseSelectOptions(question.options, question.placeHolder);
                }
                switch (question.controlType) {
                    case 'textbox':
                        questions.push(new def.TextboxQuestion(question));
                        break;
                    case 'dropdown':
                        questions.push(new def.DropdownQuestion(question));
                        break;
                    case 'textarea':
                        questions.push(new def.TextareaQuestion(question));
                        break;
                    case 'editor':
                        questions.push(new def.EditorQuestion(question));
                        break;
                    case 'text':
                        questions.push(new def.TextQuestion(question));
                        break;
                    case 'fileViewer':
                        questions.push(new def.FileViewerQuestion(question));
                        break;
                    case 'icon':
                        questions.push(new def.IconQuestion(question));
                        break;
                    case 'button':
                        questions.push(new def.ButtonQuestion(question));
                        break;
                    case 'radio':
                        questions.push(new def.RadioQuestion(question));
                        break;
                    case 'checkbox':
                        questions.push(new def.CheckboxQuestion(question));
                        break;
                    case 'multiselect':
                        questions.push(new def.MultiselectQuestion(question));
                        break;
                    case 'switch':
                        questions.push(new def.SwitchQuestion(question));
                        break;
                    case 'spinner':
                        questions.push(new def.SpinnerQuestion(question));
                        break;
                    case 'chips':
                        questions.push(new def.ChipsQuestion(question));
                        break;
                    case 'selectbutton':
                        questions.push(new def.SelectButtonQuestion(question));
                        break;
                    case 'autocomplete':
                        if (question.multiple) {
                            questions.push(new def.AutoCompleteMultiQuestion(question));
                        }
                        else {
                            questions.push(new def.AutoCompleteQuestion(question));
                        }
                        break;
                    case 'calendar':
                        questions.push(new def.CalendarQuestion(question));
                        break;
                    case 'upload':
                        questions.push(new def.UploadQuestion(question));
                        break;
                    default:
                        console.info("question type undefined: " + question.controlType);
                }
            });
            return questions.sort(function (a, b) { return a.order - b.order; });
        };
        this.parseSelectOptions = function (options, placeHolder) {
            var parsedOptions = [];
            if (placeHolder) {
                parsedOptions.push({ label: placeHolder, value: '' });
            }
            if (options.length === 1) {
                /**  why is there here ?  guy 24/02/2017 */
                // retrieve outsourced options
                // let list: any = options[0];
                // options = [];
            }
            if (options.length > 0) {
                each(options, function (option) {
                    var val = option.value ? option.value : option;
                    if (option.label) {
                        parsedOptions.push({ label: option.label, value: val });
                    }
                    else {
                        parsedOptions.push({ label: val, value: val });
                    }
                });
            }
            return parsedOptions;
        };
    }
    DFFormService.prototype.getStoredForm = function (formName, _reset) {
        return cloneDeep(this.storedForms[formName]);
    };
    DFFormService.prototype.setStoredForm = function (formName, formData) {
        this.storedForms[formName] = cloneDeep(formData);
    };
    DFFormService.prototype.getFormProperties = function (url) {
        return this.http.httpGet(url);
    };
    DFFormService.prototype.getQuestionGroups = function (body) {
        var _this = this;
        var groups = [];
        each(body.groups, function (group) {
            group.questions = _this.parseGroup(group.questions);
            groups.push(new def.QuestionGroup(group));
        });
        return groups;
    };
    DFFormService.prototype.filterOptions = function (options, parentValue, placeHolder) {
        var filteredOptions = [];
        if (options.length > 0) {
            each(options, function (option) {
                if (option.group === parentValue || !parentValue) {
                    filteredOptions.push(option);
                }
            });
        }
        return this.parseSelectOptions(filteredOptions, placeHolder);
    };
    DFFormService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    DFFormService.ctorParameters = function () { return [
        { type: HttpService, },
    ]; };
    return DFFormService;
}());
export { DFFormService };
