/*tslint:disable no-use-before-declare*/
'use strict';
import { Injectable } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
var DFQuestionControlService = /** @class */ (function () {
    function DFQuestionControlService() {
        var _this = this;
        this.toFormGroup = function (questionGroups, _disabled1, validations, allFields, allFieldNames, messages) {
            var formGroup = {};
            if (questionGroups.length > 0) {
                questionGroups.forEach(function (group) {
                    group.questions.forEach(function (question) {
                        /** guy 2/03/2017 why was disabled set to disabled1 ?  */
                        var validators = [];
                        if (question && question.required) {
                            validators.push(Validators.required);
                        }
                        if (question && question.minLength && question.minLengthVal && messages.minlength) {
                            validators.push(Validators.minLength(question.minLength));
                        }
                        if (question && question.maxLength && messages.maxlength) {
                            validators.push(Validators.maxLength(question.maxLength));
                        }
                        if (validators.length > 0) {
                            // formCtrl.setValidators(validators);
                            validations.push(question);
                        }
                        var formCtrl = new FormControl({
                            disabled: question.disabled,
                            value: question.value,
                        }, {
                            validators: validators,
                        });
                        formGroup[question.key] = formCtrl;
                        allFields.push(question);
                        allFieldNames.push(question.key);
                    });
                });
            }
            return new FormGroup(formGroup);
        };
        this.setFormErrors = function (validations) {
            var formErrors = [];
            if (validations.length > 0) {
                validations.forEach(function (field) {
                    formErrors.push(field.key, '');
                });
            }
            return formErrors;
        };
        this.changeFieldState = function (form, question, newState) {
            var q = form.get(question.key);
            if (!q) {
                return;
            }
            switch (newState) {
                case 'disable':
                    q.disable(true);
                    break;
                case 'enable':
                    q.enable(true);
                    break;
                default:
                    break;
            }
        };
        this.getQuestion = function (key, formQuestions) {
            for (var _i = 0, formQuestions_1 = formQuestions; _i < formQuestions_1.length; _i++) {
                var q = formQuestions_1[_i];
                if (q.key === key) {
                    return q;
                }
            }
        };
        this.parseMessage = function (message, field) {
            /*tslint:disable no-parameter-reassignment*/
            if (message) {
                var matches = _this.getMatch(message);
                matches.forEach(function (match) {
                    message = message.replace(match, field.getProperty(_this.strip(match)));
                });
            }
            return message;
        };
        this.getMatch = function (str) {
            var exp = new RegExp('\\[[^\\]]*]', 'g');
            return str.match(exp);
        };
        this.strip = function (str) {
            return str.substring(1, str.length - 1);
        };
    }
    DFQuestionControlService.prototype.getFormErrors = function (form, _questions, validations, _fileName, messages) {
        var _this = this;
        var formErrors = [];
        if (validations.length > 0) {
            validations.forEach(function (field) {
                // clear previous error message (if any)
                formErrors[field.key] = '';
                var control = form.get(field.key);
                if (control && control.dirty && !control.valid) {
                    for (var key in control.errors) {
                        if (messages && messages[key]) {
                            formErrors[field.key] += _this.parseMessage(messages[key], field) + " ";
                        }
                    }
                }
            });
        }
        return formErrors;
    };
    DFQuestionControlService.prototype.changeFormState = function (form, questions, newState) {
        var _this = this;
        questions.forEach(function (question) {
            _this.changeFieldState(form, question, newState);
        });
    };
    DFQuestionControlService.prototype.parseCondition = function (condition1) {
        var _this = this;
        var fields1 = this.getMatch(condition1);
        var fields2 = [];
        if (fields1) {
            fields1.forEach(function (field) {
                /*tslint:disable no-parameter-reassignment*/
                condition1 = condition1.replace(field, "this.form.value." + _this.strip(field));
                fields2.push(_this.strip(field));
            });
        }
        return { condition: condition1, fields: fields2 };
    };
    DFQuestionControlService.prototype.getDependents = function (question, fieldsToWatch, dependents, formQuestions) {
        var _this = this;
        question.dependency.forEach(function (dep) {
            if (dep.condition) {
                var parsedCondition = _this.parseCondition(dep.condition);
                dep.condition = parsedCondition.condition;
            }
            fieldsToWatch.push(dep.watch);
            dependents.push({
                condition: dep.condition,
                disable: dep.disable,
                field: _this.getQuestion(dep.key, formQuestions),
                keepValue: dep.keepValue,
                type: dep.type,
            });
        });
        return fieldsToWatch;
    };
    DFQuestionControlService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    DFQuestionControlService.ctorParameters = function () { return []; };
    return DFQuestionControlService;
}());
export { DFQuestionControlService };
