'use strict';
import { Injectable } from '@angular/core';
import { HttpService } from 'io-ng-util/lib/common/http.service';
import { Store } from '@ngrx/store';
import { Env } from 'io-ng-util/lib/common/env';
import { Router } from '@angular/router';
import { default as isDate } from 'lodash-es/isDate';
import { default as isObject } from 'lodash-es/isObject';
import { default as merge } from 'lodash-es/merge';
import { default as each } from 'lodash-es/each';
var DFStoreFunctions = /** @class */ (function () {
    function DFStoreFunctions(store, httpService, env, router) {
        var _this = this;
        this.store = store;
        this.httpService = httpService;
        this.env = env;
        this.router = router;
        this.siteError = function (err) {
            _this.store.dispatch({
                payload: err,
                type: 'NEW_ERROR',
            });
        };
        this.toSlack = function (url, storeName, err) {
            _this.httpService.httpPost('/api/slack/error/msg', {
                msg: " error => user => " + _this.env.user.email + " | msg => " + err._body + " |\n                    url => " + url + " | store => " + storeName + " ",
            })
                .subscribe(function () {
                console.info('$store.function (http post to slack) => Slack sent');
            }, function (error) {
                console.info('$store.function (http post to slack error) => Slack error', error);
            });
        };
        this.OA = function (A, B) {
            if (A === undefined || A === null) {
                if (B === undefined || B === null) {
                    return {};
                }
                else {
                    return B;
                }
            }
            return Object.assign({}, A, B);
        };
        this.OAA = function (A, B) {
            if (A === undefined || A === null) {
                if (B === undefined || B === null) {
                    return {};
                }
                else {
                    return B;
                }
            }
            return Object.assign(A, B);
        };
        this.r_qP = function (route) {
            var obj = {};
            route.queryParams.forEach(function (params) {
                var t = params;
                for (var p in t) {
                    if (t.hasOwnProperty(p)) {
                        obj = Object.assign({}, obj, (_a = {},
                            _a[p] = t[p],
                            _a));
                    }
                }
                var _a;
            });
            return obj;
        };
        this.r_PP = function (route) {
            var obj = {};
            route.parent.params.forEach(function (params) {
                var tt = params;
                for (var p in tt) {
                    if (tt.hasOwnProperty(p)) {
                        obj = Object.assign({}, obj, (_a = {},
                            _a[p] = tt[p],
                            _a));
                    }
                }
                var _a;
            });
            return obj;
        };
        this.r_P = function (route) {
            var obj = {};
            route.params.forEach(function (params) {
                var ttt = params;
                for (var p in ttt) {
                    if (ttt.hasOwnProperty(p)) {
                        obj = Object.assign({}, obj, (_a = {},
                            _a[p] = ttt[p],
                            _a));
                    }
                }
                var _a;
            });
            return obj;
        };
        this.r_getStore = function (store) {
            var tree = _this.router.parseUrl(_this.router.url);
            var storeName = tree.root.children.primary.segments.join('-');
            var params = tree.queryParams;
            for (var p in params) {
                if (params.hasOwnProperty(p)) {
                    storeName += "-" + params[p];
                }
            }
            return store + "." + storeName;
        };
        this.getNbr = function (records) {
            if (records.length === 0) {
                return 0;
            }
            else if (records[0] && records[0].nbr) {
                return records[0].nbr;
            }
            else {
                return records.length;
            }
        };
        this.getEmpty = function (records) {
            if (records.length === 0) {
                return true;
            }
            else {
                return false;
            }
        };
        this.flatten = function (val) {
            each(val, function (v) {
                if (!isDate(val) && isObject(v)) {
                    merge(val, v);
                }
            });
            return val;
        };
        return;
    }
    DFStoreFunctions.prototype.initStore = function (store, query) {
        this.store.dispatch({
            payload: {
                loaded: false,
                query: query,
                store: store,
            },
            type: 'ADD_STORE',
        });
    };
    DFStoreFunctions.prototype.resetStore = function (store) {
        this.store.dispatch({
            payload: {
                store: store,
            },
            type: 'RESET_STORE',
        });
    };
    DFStoreFunctions.prototype.resetFormStore = function (store) {
        this.store.dispatch({
            payload: {
                store: store,
            },
            type: 'RESET_FORM_STORE',
        });
    };
    DFStoreFunctions.prototype.updateForm = function (_store, payload) {
        this.store.dispatch({
            payload: payload,
            type: 'ADD_FORM_DATA',
        });
    };
    DFStoreFunctions.prototype.loadForm = function (params, locals, storeName) {
        var _this = this;
        var url = locals.base
            ? locals.base + locals.url
            : locals.url;
        var values = locals.flatten === undefined
            ? true
            : locals.flatten;
        this.httpService.httpPost(url, params, values)
            .subscribe(function (records) {
            var vals = records;
            /** data can come in also in array , in that case we turn into an object  */
            if (Array.isArray(records)) {
                vals = records[0];
            }
            if (vals === undefined) {
                vals = {};
            }
            var payload = {
                error: false,
                form: {
                    name: locals.formName,
                    valid: false,
                },
                // in case we want to load for one form
                loaded: true,
                store: storeName,
                target: [],
                values: vals,
            };
            _this.store.dispatch({
                payload: payload,
                type: 'ADD_FORM_DATA',
            });
            return true;
        }, function (err) {
            var payload = {
                error: true,
                form: {
                    name: locals.formName,
                    valid: false,
                },
                loaded: false,
                store: storeName,
                target: [storeName],
                values: [],
            };
            _this.store.dispatch({
                payload: payload,
                type: 'ADD_FORM_DATA',
            });
            _this.siteError(err);
            _this.toSlack(url, storeName, err);
        });
    };
    DFStoreFunctions.prototype.loadStore = function (params, locals, storeName) {
        var _this = this;
        var url = locals.base
            ? locals.base + locals.url
            : locals.url;
        var values = locals.flatten === undefined ? true : locals.flatten;
        this.httpService.httpPost(url, params, values)
            .subscribe(function (records) {
            var payload = {
                empty: _this.getEmpty(records),
                error: false,
                loaded: true,
                query: params.query,
                records: records,
                store: storeName,
                totalRecords: _this.getNbr(records),
            };
            _this.store.dispatch({
                payload: payload,
                type: locals.type,
            });
        }, function (err) {
            var payload = {
                empty: true,
                error: true,
                loaded: true,
                query: {},
                records: [],
                store: storeName,
                totalRecords: 0,
            };
            _this.store.dispatch({
                payload: payload,
                type: locals.type,
            });
            _this.siteError(err);
            _this.toSlack(url, storeName, err);
        });
    };
    DFStoreFunctions.prototype.loadNode = function (params, locals, storeName, node) {
        var _this = this;
        var url = locals.base + locals.url;
        this.httpService.httpPost(url, params)
            .subscribe(function (records) {
            node.children = records;
        }, function (err) {
            _this.httpService.httpPost('/api/slack/error/msg', {
                msg: " error => user => " + _this.env.user.email + " | msg => " + err._body + " |\n                      url => " + url + " | store => " + storeName + " ",
            })
                .subscribe(function () {
                console.info('$store.function (http post to slack) => Slack sent');
            }, function (error) {
                console.info('$store.function (http post to slack error) => Slack error', error);
            });
        });
    };
    DFStoreFunctions.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    DFStoreFunctions.ctorParameters = function () { return [
        { type: Store, },
        { type: HttpService, },
        { type: Env, },
        { type: Router, },
    ]; };
    return DFStoreFunctions;
}());
export { DFStoreFunctions };
