'use strict';
import { tmpl } from './templates/df.menu.dropdown.pug';
import { default as each } from 'lodash-es/each';
import { Store } from '@ngrx/store';
import { DFStoreFunctions } from '../construct/services/store/df.store.functions';
import { DFFormService } from '../construct/services/df.form.service';
import { DFCommonService } from '../construct/services/df.common.api';
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input } from '@angular/core';
import 'rxjs/add/operator/take';
var DFMenuDropDownComponent = /** @class */ (function () {
    function DFMenuDropDownComponent(cd, DFCS, DSF, formService, store) {
        var _this = this;
        this.cd = cd;
        this.DFCS = DFCS;
        this.DSF = DSF;
        this.formService = formService;
        this.store = store;
        this.formSpecs = {};
        this.awaitData = function (store, items) {
            var storeName = _this.DSF.r_getStore(store);
            _this.AStore = _this.store
                .select(function (s) { return s.coverage.states[storeName]; })
                .debounceTime(100)
                .subscribe(function (data) {
                if (data !== undefined) {
                    _this.handleRules(data.values, items);
                }
            });
            return;
        };
        this.handleRules = function (data, items) {
            _this.data = data;
            each(items, function (item) {
                // we now have an item that equals a drop down value
                if (!item.rules) {
                    return; // we can stop here as there's no rule
                }
                each(item.rules, function (rule) {
                    // we now have the role
                    switch (rule.type) {
                        case 'hide-when':
                            // the rule is to hide
                            item.hidden = _this.evalStr(rule.rule);
                            _this.check();
                            break;
                        default:
                            return;
                    }
                });
            });
        };
        this.onAction = function (action) {
            _this.DFCS.formChanged(action);
        };
        this.evalStr = function (str) {
            try {
                return eval(str);
            }
            catch (err) {
                console.debug('$df.menu.dropdown (handleRules) error', err);
                return false;
            }
        };
        this.check = function () {
            _this.cd.markForCheck();
        };
    }
    DFMenuDropDownComponent.prototype.ngOnInit = function () {
        if (this.config) {
            this.getForm();
        }
    };
    DFMenuDropDownComponent.prototype.getForm = function () {
        var _this = this;
        var formName = this.config.formName;
        if (formName === undefined) {
            return console.info("$df.menu.dropdown.component (getForm) => no formName defined");
        }
        var storedForm = this.formService.getStoredForm(formName);
        if (storedForm) {
            this.formSpecs = storedForm;
            console.info("$df.menu.dropdown.component (getForm) => reusing " + formName);
            this.check();
        }
        else {
            this.FSStore = this.formService
                .getFormProperties("api/forms/operator/getFormQuestions?form=" + formName)
                .take(1) // only subscribe once
                .subscribe(function (results) {
                _this.formSpecs = results;
                _this.formService.setStoredForm(formName, results);
                _this.check();
            }, function (error) {
                console.info('$df.menu.dropdown.component (getForm) => calling ', error);
            });
        }
        /** this is different , it checks if the form has */
        if (this.formSpecs && this.formSpecs.hasRules) {
            this.awaitData(this.formSpecs.store, this.formSpecs.items);
        }
    };
    DFMenuDropDownComponent.prototype.ngOnDestroy = function () {
        /** a security to ensure that we always start with a not loaded component
                 * usecase when the localdata is not destroyed
                 */
        if (this.FSStore !== undefined) {
            this.FSStore.unsubscribe();
        }
        if (this.AStore !== undefined) {
            this.AStore.unsubscribe();
        }
        console.info("$df.menu.dropdown.component (ngOnDestroy) => destroying subscription " + this.config.formName);
    };
    DFMenuDropDownComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    selector: 'df-menu-dropdown',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFMenuDropDownComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: DFCommonService, },
        { type: DFStoreFunctions, },
        { type: DFFormService, },
        { type: Store, },
    ]; };
    DFMenuDropDownComponent.propDecorators = {
        "config": [{ type: Input },],
    };
    return DFMenuDropDownComponent;
}());
export { DFMenuDropDownComponent };
