'use strict';
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input } from '@angular/core';
import { tmpl } from './templates/df.section.multi.pug';
import { DFFormService } from '../construct/services/df.form.service';
import { default as each } from 'lodash-es/each';
import 'rxjs/add/operator/take';
import { DFStoreFunctions } from '../construct/services/store/df.store.functions';
import { Store } from '@ngrx/store';
var DFSectionMultiComponent = /** @class */ (function () {
    function DFSectionMultiComponent(cd, DSF, DFFS, store) {
        var _this = this;
        this.formSpecs = {};
        this.getForm = function (form) {
            return new Promise(function (resolve, reject) {
                var formName = form === undefined
                    ? _this.config.formName
                    : form;
                if (formName === undefined) {
                    console.info("$df.section.multi.component (getForm) => no formName defined");
                    return reject(false);
                }
                var storedForm = _this.DFFS.getStoredForm(formName);
                if (storedForm) {
                    console.info("$df.section.multi.component (getForm) => reusing " + formName);
                    return resolve(storedForm);
                }
                _this.FSStore = _this.DFFS
                    .getFormProperties("api/forms/operator/getFormQuestions?form=" + formName)
                    .take(1) // only subscribe once
                    .subscribe(function (results) {
                    each(results.forms, function (f) {
                        if (f.type === 'dynForm') {
                            var questionGroups = _this.DFFS.getQuestionGroups(f);
                            _this.DFFS.setStoredForm(f.id, { groups: questionGroups, specs: f });
                        }
                        else if (f.type === 'dynSection') {
                            _this.DFFS.setStoredForm(f.id, f);
                        }
                        else {
                            _this.DFFS.setStoredForm(f.id, { specs: f });
                        }
                        console.info("$df.section.multi.component (getForm) => loaded " + f.id);
                    });
                    _this.DFFS.setStoredForm(formName, results);
                    resolve(results);
                }, function (error) {
                    console.info('$df.section.multi.component (getForm) => calling ', error);
                    reject(error);
                });
            });
        };
        this.getValues = function (fieldsetValues, i) {
            var values = _this.config.values;
            var fv = fieldsetValues;
            var t = false;
            if (values) {
                fv = __assign({}, fieldsetValues, { values: values });
                t = true;
            }
            var params = _this.config.params;
            if (params && fv.locals && fv.locals.params) {
                var lparams = _this.DSF.OAA(fv.locals.params, _this.config.params);
                _this.DSF.OAA(fv.locals.params, lparams);
                t = true;
            }
            if (fv.fieldset && fv.fieldset.dynamic) {
                _this.awaitTemplate(fv.fieldset.dynamic, i);
            }
            if (t) {
                return fv;
            }
            return t;
        };
        this.awaitTemplate = function (store, i) {
            var storeName = _this.DSF.r_getStore(store);
            _this.AStore = _this.store
                .select(function (s) { return s.coverage.states[storeName]; })
                .subscribe(function (data) {
                if (data && data.values && data.values.template) {
                    _this.getForm(data.values.template)
                        .then(function (res) {
                        _this.formSpecs.fieldsets[i] = {};
                        _this.formSpecs.fieldsets[i] = res.fieldsets[0];
                        _this.cd.markForCheck();
                    });
                }
            });
            return;
        };
        this.check = function () {
            _this.cd.markForCheck();
        };
        this.cd = cd;
        this.DSF = DSF;
        this.DFFS = DFFS;
        this.store = store;
    }
    DFSectionMultiComponent.prototype.ngOnInit = function () {
        var _this = this;
        if (this.config) {
            this.getForm(undefined)
                .then(function (results) {
                _this.formSpecs = results;
                each(_this.formSpecs.fieldsets, function (fieldsetValues, i) {
                    var t = _this.getValues(fieldsetValues, i);
                    if (t) {
                        _this.formSpecs.fieldsets[i] = t;
                    }
                });
                _this.check();
            });
        }
    };
    DFSectionMultiComponent.prototype.ngOnDestroy = function () {
        /** a security to ensure that we always start with a not loaded component
                 * usecase when the localdata is not destroyed
                 */
        if (this.FSStore !== undefined) {
            this.FSStore.unsubscribe();
        }
        if (this.AStore !== undefined) {
            this.AStore.unsubscribe();
        }
        console.info("$df.section.multi.component (ngOnDestroy) => destroying subscription " + this.config.formName);
    };
    DFSectionMultiComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    selector: 'df-section-multi',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFSectionMultiComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: DFStoreFunctions, },
        { type: DFFormService, },
        { type: Store, },
    ]; };
    DFSectionMultiComponent.propDecorators = {
        "config": [{ type: Input },],
    };
    return DFSectionMultiComponent;
}());
export { DFSectionMultiComponent };
