'use strict';
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, EventEmitter, Input, Output, } from '@angular/core';
import { DFFormService } from '../construct/services/df.form.service';
import { tmpl } from './templates/df.standard.form.pug';
import 'rxjs/add/operator/take';
var DFStandardFormComponent = /** @class */ (function () {
    function DFStandardFormComponent(cd, formService) {
        var _this = this;
        this.cd = cd;
        this.formService = formService;
        this.result = new EventEmitter();
        this.questionGroups = [];
        this.check = function () {
            _this.cd.markForCheck();
        };
    }
    DFStandardFormComponent.prototype.ngOnInit = function () {
        if (this.locals.formName) {
            this.getForm();
        }
    };
    DFStandardFormComponent.prototype.ngOnChanges = function (changes) {
        var change = changes;
        if (this.formSpecs === undefined) {
            return;
        }
        if (change.changed) {
            this.getForm();
        }
    };
    DFStandardFormComponent.prototype.getForm = function () {
        var _this = this;
        var formName = this.locals.formName;
        if (formName === undefined) {
            return console.info("$df.standard.form.component (getForm) => no formName defined");
        }
        var storedForm = this.formService.getStoredForm(formName, this.locals.reset || false);
        if (storedForm) {
            this.formSpecs = storedForm.specs;
            this.questionGroups = storedForm.groups;
            this.formSpecs.readmode = this.locals.readmode !== undefined ? this.locals.readmode
                : (this.formSpecs.readmode || false);
            this.formSpecs.disabled = !!this.locals.disabled;
            this.formSpecs.submitted = !!this.locals.submitted;
            console.info("$df.standard.form.component (getForm) => reusing " + formName);
            this.check();
        }
        else {
            this.FSStore = this.formService
                .getFormProperties("api/forms/operator/getFormQuestions?form=" + formName)
                .take(1) // only subscribe once
                .subscribe(function (qDefs) {
                _this.formSpecs = qDefs;
                _this.formSpecs.readmode = _this.locals.readmode !== undefined ? _this.locals.readmode
                    : _this.formSpecs.readmode || false;
                _this.formSpecs.disabled = !!_this.locals.disabled;
                _this.formSpecs.disableOnSubmit = true;
                _this.formSpecs.submitted = !!_this.locals.submitted;
                _this.formSpecs.editEnabled = true;
                _this.formSpecs.deleteEnabled = true;
                _this.questionGroups = _this.formService.getQuestionGroups(qDefs);
                _this.formService.setStoredForm(formName, { groups: _this.questionGroups, specs: _this.formSpecs });
                _this.check();
            }, function (error) {
                console.info('$df.standard.form.component (getForm) => calling ', error);
            });
        }
    };
    DFStandardFormComponent.prototype.ngOnDestroy = function () {
        /** a security to ensure that we always start with a not loaded component
                 * usecase when the localdata is not destroyed
                 */
        if (this.FSStore !== undefined) {
            this.FSStore.unsubscribe();
        }
        console.info("$df.standard.form.component (ngOnDestroy) => destroying subscription " + this.locals.formName);
    };
    DFStandardFormComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    selector: 'df-standard-form',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFStandardFormComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: DFFormService, },
    ]; };
    DFStandardFormComponent.propDecorators = {
        "locals": [{ type: Input },],
        "values": [{ type: Input },],
        "changed": [{ type: Input },],
        "result": [{ type: Output },],
    };
    return DFStandardFormComponent;
}());
export { DFStandardFormComponent };
