'use strict';
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { DFFormService } from '../construct/services/df.form.service';
import { tmpl } from './templates/df.table.list.pug';
import 'rxjs/add/operator/take';
var DFTableListComponent = /** @class */ (function () {
    function DFTableListComponent(formService) {
        this.formService = formService;
        this.result = new EventEmitter();
    }
    DFTableListComponent.prototype.ngOnInit = function () {
        this.getForm();
    };
    DFTableListComponent.prototype.getForm = function () {
        var _this = this;
        var formName = this.locals.formName;
        if (formName === undefined) {
            return console.info("$df.table.list.component (getForm) => no formName defined");
        }
        var storedForm = this.formService.getStoredForm(formName) || false;
        if (typeof storedForm === 'object') {
            console.info("$df.table.list.component (getForm) => reusing " + formName);
            this.formSpecs = storedForm.specs;
        }
        else {
            console.info("$df.table.list.component (getForm) => calling " + this.locals.formName);
            this.FSStore = this.formService
                .getFormProperties("api/forms/operator/getFormQuestions?form=" + formName)
                .take(1) // only subscribe once
                .subscribe(function (qDefs) {
                _this.formSpecs = qDefs; // specs are used by the template
                _this.formService.setStoredForm(formName, { specs: _this.formSpecs }); // store form for reuse
            }, function (error) {
                console.info("$df.table.list.component (getForm) => error calling " + formName, error);
            });
        }
    };
    DFTableListComponent.prototype.getResult = function (results) {
        this.result.emit(results);
    };
    DFTableListComponent.prototype.ngOnDestroy = function () {
        /** a security to ensure that we always start with a not loaded component
                 * usecase when the localdata is not destroyed
                 */
        if (this.FSStore !== undefined) {
            this.FSStore.unsubscribe();
        }
        console.info("$df.table.list.component (ngOnDestroy) => destroying subscription " + this.locals.formName);
    };
    DFTableListComponent.decorators = [
        { type: Component, args: [{
                    selector: 'df-table-list',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFTableListComponent.ctorParameters = function () { return [
        { type: DFFormService, },
    ]; };
    DFTableListComponent.propDecorators = {
        "result": [{ type: Output },],
        "formLocals": [{ type: Input },],
        "locals": [{ type: Input },],
        "params": [{ type: Input },],
        "storeData": [{ type: Input },],
        "values": [{ type: Input },],
    };
    return DFTableListComponent;
}());
export { DFTableListComponent };
