'use strict';
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input } from '@angular/core';
import { tmpl } from './templates/df.tabs.section.multi.pug';
import { DFFormService } from '../construct/services/df.form.service';
import { default as each } from 'lodash-es/each';
import 'rxjs/add/operator/take';
var DFTabsSectionMultiComponent = /** @class */ (function () {
    function DFTabsSectionMultiComponent(cd, formService) {
        var _this = this;
        this.cd = cd;
        this.formService = formService;
        this.formSpecs = {};
        this.check = function () {
            _this.cd.markForCheck();
        };
    }
    DFTabsSectionMultiComponent.prototype.ngOnInit = function () {
        if (this.config) {
            this.getForm();
        }
    };
    DFTabsSectionMultiComponent.prototype.getForm = function () {
        var _this = this;
        var formName = this.config.formName;
        if (formName === undefined) {
            return console.info("$df.tabs.section.multi.component (getForm) => no formName defined");
        }
        var storedForm = this.formService.getStoredForm(formName);
        if (storedForm) {
            this.formSpecs = storedForm;
            console.info("$df.tabs.section.multi.component (getForm) => reusing " + formName);
            this.check();
        }
        else {
            this.FSStore = this.formService
                .getFormProperties("api/forms/operator/getFormQuestions?form=" + formName)
                .take(1) // only subscribe once
                .subscribe(function (results) {
                each(results.forms, function (f) {
                    _this.formService.setStoredForm(f.id, f);
                    console.info("$df.tabs.section.multi.component (getForm) => loaded " + f.id);
                });
                _this.formSpecs = results;
                _this.formService.setStoredForm(formName, results);
                _this.check();
            }, function (error) {
                console.info('$df.tabs.section.multi.component (getForm) => calling ', error);
            });
        }
    };
    DFTabsSectionMultiComponent.prototype.ngOnDestroy = function () {
        /** a security to ensure that we always start with a not loaded component
                 * usecase when the localdata is not destroyed
                 */
        if (this.FSStore !== undefined) {
            this.FSStore.unsubscribe();
        }
        console.info("$df.tabs.section.multi.component (ngOnDestroy) => destroying subscription " + this.config.formName);
    };
    DFTabsSectionMultiComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    selector: 'df-tabs-section-multi',
                    template: tmpl,
                },] },
    ];
    /** @nocollapse */
    DFTabsSectionMultiComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: DFFormService, },
    ]; };
    DFTabsSectionMultiComponent.propDecorators = {
        "config": [{ type: Input },],
    };
    return DFTabsSectionMultiComponent;
}());
export { DFTabsSectionMultiComponent };
